@@ -4,11 +4,17 @@
 #define strcasecmp _stricmp
 #endif
 
+typedef struct {
+    int error;
+    int keep_alive;
+} RequestState;
+
 typedef struct {
     PyObject* headers;
     char remote_addr[17];
     llhttp_t parser;
     uv_buf_t response_buffer;
+    RequestState state;
 } Request;
 
 typedef struct {
