@@ -2,13 +2,16 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <uv.h>
+#include "uv.h"
 #include "llhttp.h"
 
 #include "server.h"
 #include "request.h"
 #include "constants.h"
 
+static const char* BAD_REQUEST = "HTTP/1.1 400 Bad Request\r\n\r\n";
+static const char* INTERNAL_ERROR = "HTTP/1.1 500 Internal Server Error\r\n\r\n";
+
 void logger(char* message) {
     if (LOGGING_ENABLED)
         fprintf(stdout, ">>> %s\n", message);
@@ -19,6 +22,18 @@ void close_cb(uv_handle_t* handle) {
     free(handle);
 }
 
+void shutdown_cb(uv_shutdown_t* req, int status) {
+    uv_handle_t* handle = (uv_handle_t*) req->handle;
+    if (!uv_is_closing(handle))
+        uv_close(handle, close_cb);
+    free(req);
+}
+
+void close_connection(uv_stream_t* handle) {
+    uv_shutdown_t* shutdown = malloc(sizeof(uv_shutdown_t));
+    uv_shutdown(shutdown, handle, shutdown_cb);
+}
+
 void write_cb(uv_write_t* req, int status) {
     if (status) {
         fprintf(stderr, "Write error %s\n", uv_strerror(status));
@@ -28,33 +43,68 @@ void write_cb(uv_write_t* req, int status) {
     free(write_req);
 }
 
+void send_error(write_req_t* req, uv_stream_t* handle, const char* error_string) {
+    char* error = malloc(strlen(error_string) + 1);
+    strcpy(error, error_string);
+    req->buf = uv_buf_init(error, strlen(error));
+    uv_write((uv_write_t*)req, handle, &req->buf, 1, write_cb);
+    close_connection(handle);
+}
+
+void send_response(write_req_t* req, uv_stream_t* handle, Request* request) {
+    uv_buf_t response = request->response_buffer;
+    req->buf = uv_buf_init(response.base, response.len);
+    uv_write((uv_write_t*)req, handle, &req->buf, 1, write_cb);
+    if (!request->state.keep_alive)
+        close_connection(handle);
+}
+
+
 void read_cb(uv_stream_t* handle, ssize_t nread, const uv_buf_t* buf) {
     client_t* client = (client_t*)handle->data;
 
     Request* request = malloc(sizeof(Request));
+    request->state.keep_alive = 0;
+    request->state.error = 0;
     strcpy(request->remote_addr, client->remote_addr);
     client->parser.data = request;
+    write_req_t* req = (write_req_t*)malloc(sizeof(write_req_t));
 
-    if (nread >= 0) {
+    if (nread > 0) {
         enum llhttp_errno err = llhttp_execute(&client->parser, buf->base, nread);
         if (err == HPE_OK) {
             logger("Successfully parsed");
-            write_req_t* req = (write_req_t*)malloc(sizeof(write_req_t));
-            req->buf = uv_buf_init(request->response_buffer.base, request->response_buffer.len);
-            uv_write((uv_write_t*)req, handle, &req->buf, 1, write_cb);
+            if (request->response_buffer.len > 0)
+                send_response(req, handle, request);
+            else if (request->state.error)
+                send_error(req, handle, INTERNAL_ERROR);
+            else
+                send_error(req, handle, BAD_REQUEST);
         }
         else {
             fprintf(stderr, "Parse error: %s %s\n", llhttp_errno_name(err), client->parser.reason);
+            send_error(req, handle, BAD_REQUEST);
         }
     }
-    else {
-        if (nread != UV_EOF && nread != UV_ECONNRESET)
+    if (nread < 0) {
+        uv_read_stop(handle);
+
+        if (nread == UV_ECONNRESET) {
+            close_connection(handle);
+        }
+        else if (nread != UV_EOF) {
             fprintf(stderr, "Read error %s\n", uv_err_name(nread));
-        uv_close((uv_handle_t*)handle, close_cb);
+            close_connection(handle);
+        }
     }
     free(request);
-    free(buf->base);
     llhttp_reset(&client->parser);
+    
+    if (buf->base)
+        free(buf->base);
+    
+    if (PyErr_Occurred())
+        PyErr_Clear();
 }
 
 void alloc_cb(uv_handle_t* handle, size_t suggested_size, uv_buf_t* buf) {
@@ -144,4 +194,4 @@ PyObject* run_server(PyObject* self, PyObject* args) {
     PyArg_ParseTuple(args, "Osiii", &wsgi_app, &host, &port, &backlog, &LOGGING_ENABLED);
     main();
     Py_RETURN_NONE;
-}
\ No newline at end of file
+}
